FUNCTION plcount
PARAMETERS pt_str, pt_wid, pt_aname, pt_font, pt_fsize, ;
          pt_fstyle
PRIVATE pt_cur_wid, pt_chop, pt_word, pt_isarr, pt_xx

*  plcount(<expC1>,<expN1>[,<@ArrayName>][,<expC2>,;       *
*          <expN2>,<expC3>])                               *
*                                                          *
*  Note: Function returns the number of lines of text of   *
*  a specified width in an (optionally) specified font in  *
*  a text string.  Optionally, the function stores the     *
*  lines to an array.                                      *

EXTERNAL ARRAY pt_aname

***  do we have an array to store the lines to?

m.pt_isarr = type('pt_aname[1]') # 'U'

IF m.pt_isarr

  IF parameters() = 3        &&  not fonts passed
    m.pt_font = wfont(1)
    m.pt_fsize = wfont(2)
    m.pt_fstyle = wfont(3)
  ENDIF

ELSE
  
  IF parameters() = 2
    m.pt_font = wfont(1)
    m.pt_fsize = wfont(2)
    m.pt_fstyle = wfont(3)
  ELSE
    m.pt_fstyle = m.pt_fsize
    m.pt_fsize = m.pt_font
    m.pt_font = pt_aname
  ENDIF

ENDIF

m.pt_xx = 0       &&  initialize return value

IF type('m.pt_str') = 'C' .and. ;
   type('m.pt_wid') = 'N' .and. ;
   m.pt_wid >= 8 .and. ;
   type('m.pt_isarr') = 'L' .and. ;
   type('m.pt_font') = 'C' .and. ;
   type('m.pt_fsize') = 'N' .and. ;
   type('m.pt_fstyle') = 'C'

  m.pt_cur_wid = set('memowidth')
  SET MEMOWIDTH TO m.pt_wid     &&  set new width
  m.pt_xx = 1

  DO CASE

***  Windows and proportionally-spaced fonts

    CASE _windows .and. ;
         (txtwidth('i', m.pt_font, 10) # 1 .or. ;
          txtwidth('W', m.pt_font, 10) # 1)

      DO WHILE .not. empty(m.pt_str)
        m.pt_str = alltrim(m.pt_str) + ' '
        m.pt_chop = mline(m.pt_str,1)

        DO CASE 

***  if text width less than line width add words

          CASE txtwidth(m.pt_chop, m.pt_font, m.pt_fsize, ;
                        m.pt_fstyle) < m.pt_wid
            m.pt_str = substr(m.pt_str, len(m.pt_chop) + 1)

            DO WHILE .t.
              m.pt_word = left(m.pt_str, ;
                             at(' ',m.pt_str) - 1)

***  watch out for carriage returns when adding 
***  words to string.  If found, break line there.

              IF chr(13) $ m.pt_word

                IF txtwidth(m.pt_chop, m.pt_font, ;
                            m.pt_fsize, m.pt_fstyle) + ;
                   txtwidth(left(m.pt_word, ;
                     at(chr(13), m.pt_word) -1), ;
                     m.pt_font,m.pt_fsize,m.pt_fstyle) <= ;
                   m.pt_wid

                  m.pt_chop = m.pt_chop + left(m.pt_word, ;
                            at(chr(13), m.pt_word)-1)
                  m.pt_str = substr(m.pt_str, at(chr(13),;
                                  m.pt_str) + 2)
                ENDIF

                EXIT
              ENDIF

***  if last word makes text width greater than 
***  the line length, do not add it and go to 
***  next line.

              IF txtwidth(m.pt_chop+m.pt_word,m.pt_font, ;
                          m.pt_fsize, m.pt_fstyle) > ;
                          m.pt_wid
                EXIT
              ELSE
                m.pt_chop = m.pt_chop + m.pt_word + ' '
                m.pt_str = ltrim(substr(m.pt_str, ;
                               len(m.pt_word) + 1))
              ENDIF

            ENDDO

***  if the text width is greater than the line
***  length, chop words off line and add them back 
***  to the string until the line fits fits.

          CASE txtwidth(m.pt_chop, m.pt_font, m.pt_fsize,;
                        m.pt_fstyle) > m.pt_wid
            m.pt_str = substr(m.pt_str, len(m.pt_chop)+1)

            DO WHILE .t.
              m.pt_word = substr(m.pt_chop, ;
                                 rat(' ',m.pt_chop))
              m.pt_str = ltrim(m.pt_word) + m.pt_str
              m.pt_chop = trim(left(m.pt_chop, ;
                               len(m.pt_chop)- ;
                               len(m.pt_word)))

***  check to see if string is now less than the width

              IF txtwidth(m.pt_chop, m.pt_font,;
                          m.pt_fsize, m.pt_fstyle) < ;
                   m.pt_wid
                EXIT
              ENDIF

            ENDDO

***  if text width equals the line width

          CASE txtwidth(m.pt_chop, m.pt_font, m.pt_fsize, ;
                        m.pt_fstyle) = m.pt_wid
            m.pt_str = substr(m.pt_str, len(m.pt_chop) + 1)

        ENDCASE

***  if storing to an array, redimension it and
***  add the new line to the last element.

        IF m.pt_isarr
          DECLARE pt_aname[m.pt_xx]
          pt_aname[m.pt_xx] = trim(m.pt_chop)
        ENDIF

        m.pt_xx = m.pt_xx + 1

      ENDDO

    CASE _windows .and. ;
         (txtwidth('i', m.pt_font, 10) = 1 .and. ;
          txtwidth('W', m.pt_font, 10) = 1)

      m.pt_xx = ptmslines()

    CASE _dos
      m.pt_xx = ptmslines()

    OTHERWISE
      m.pt_xx = ptmslines()

  ENDCASE

  SET MEMOWIDTH TO (m.pt_cur_wid)

ENDIF

RETURN(m.pt_xx - 1)

********

FUNCTION ptmslines
PRIVATE pt_yy

m.pt_xx = memlines(m.pt_str)

IF m.pt_isarr
  DECLARE pt_aname[m.pt_xx]
  _mline = 0

  FOR m.pt_yy = 1 TO m.pt_xx
    pt_aname[m.pt_yy] = mline(m.pt_str, 1, _mline)
  ENDFOR

ENDIF

RETURN(m.pt_xx + 1)